#include "server/auth.h"
#include "server/util.h"

#include "db/auth.h"

#include "jwt/jwt.h"

#include <civetweb.h>

#include <cjson/cJSON.h>

#include <assert.h>
#include <stdlib.h>
#include <string.h>

typedef struct
{
    char* token;
} delete_form_t;

static void delete_form_dtor(delete_form_t* form)
{
    if (form->token) free(form->token);
}

static int field_found(const char* key, const char* filename, char* path, size_t pathlen, void* user_data)
{
    return MG_FORM_FIELD_HANDLE_GET;
}

static int field_get(const char* key, const char* value, size_t valuelen, void* user_data)
{
    delete_form_t* form = (delete_form_t*)user_data;
    if (strcmp(key, "token") == 0) {
        form->token = kqm_strndup(value, valuelen);
        return MG_FORM_FIELD_HANDLE_ABORT;
    }
    return MG_FORM_FIELD_HANDLE_GET;
}


int delete_handler(mg_connection* conn, void* cbdata)
{
    const mg_request_info* post_body = mg_get_request_info(conn);

    if (post_body == NULL) {
        mg_printf(conn,
                  "HTTP/1.1 400 Bad Request\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n");
        mg_printf(conn, "{\"error\":\"null request\"}");
        return 1;
    }

    if (strcmp(post_body->request_method, "POST")) {
        mg_printf(conn,
                  "HTTP/1.1 405 Method Not Allowed\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n");
        mg_printf(conn, "{\"error\":\"must use post request\"}");
        return 1;
    }

    delete_form_t form = {NULL};

    mg_form_data_handler delete_callback = {
        .field_found = field_found,
        .field_get   = field_get,
        .field_store = NULL,
        .user_data   = &form,
    };

    mg_handle_form_request(conn, &delete_callback);

    if (!form.token) {
        mg_printf(conn,
                  "HTTP/1.1 400 Bad Request\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n"
                  "{\"error\":\"need token\"}");
        delete_form_dtor(&form);
        return 1;
    }

    int result = verify_token(form.token, secret);
    if (!result) {
        mg_printf(conn,
                  "HTTP/1.1 401 Unauthorized\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n"
                  "{\"error\":\"auth failed\"}");
        delete_form_dtor(&form);
        return 1;
    }

    char* user_id = get_payload(form.token);

    int flag = check_user_exists(user_id, &result);
    if (!flag) {
        mg_printf(conn,
                  "HTTP/1.1 500 Internal Server Error\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n"
                  "{\"error\":\"failed to check user existence\"}");
        delete_form_dtor(&form);
        return 1;
    } else if (!result) {
        mg_printf(conn,
                  "HTTP/1.1 404 Not Found\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n"
                  "{\"error\":\"user does not exist\"}");
        delete_form_dtor(&form);
        return 1;
    }

    flag = delete_user(user_id);
    if (!flag) {
        mg_printf(conn,
                  "HTTP/1.1 500 Internal Server Error\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n"
                  "{\"error\":\"failed to delete account\"}");
    } else {
        mg_printf(conn,
                  "HTTP/1.1 200 OK\r\n"
                  "Content-Type: application/json\r\n"
                  "Access-Control-Allow-Origin: *\r\n\r\n"
                  "{\"success\":\"delete account success\"}");
    }
    free(user_id);
    delete_form_dtor(&form);
    return 1;
}
